#ifndef _CEGOADMINTHREADPOOL_H_INCLUDED_
#define _CEGOADMINTHREADPOOL_H_INCLUDED_
///////////////////////////////////////////////////////////////////////////////
//                                                         
// CegoAdminThreadPool.h  
// ---------------------
// Cego threadpool class definition
//                                                        
// Design and Implementation by Bjoern Lemke
//
// (C)opyright 2000-2010 Bjoern Lemke
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2, or (at your option)
// any later version.
// 
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; see the file COPYING.  If not, write to
// the Free Software Foundation, 59 Temple Place - Suite 330,
// Boston, MA 02111-1307, USA.
//
// INTERFACE MODULE
//
// Class: CegoAdminThreadPool
// 
// Description:
//
// Status: QG-2.6
//
///////////////////////////////////////////////////////////////////////////////

#include <lfc/Chain.h>
#include <lfc/ListT.h>
#include <lfc/Thread.h>
#include <lfc/NetHandler.h>

#include "CegoDatabaseManager.h"
#include "CegoDbThreadPool.h"
#include "CegoLogThreadPool.h"

class CegoAdminThread;

class CegoAdminThreadPool : public Thread {
    
 public:

    enum ThreadState { READY, BUSY, CONNECTED };
  
    CegoAdminThreadPool();
    CegoAdminThreadPool(int poolLimit, CegoDatabaseManager *pDBMng, CegoDbThreadPool *pDbPool, CegoLogThreadPool *pLogPool);
    ~CegoAdminThreadPool();

    int getPoolLimit() const;
    void* job(void* arg);

    void getThreadInfo(long i, long& numRequest, ThreadState& state, Chain& action);
    void setState(long i, ThreadState state);
    void setTid(long i, long tid);
    void incNumRequest(long i);

    void dispatch();

    NetHandler* nextRequest();

    bool isTerminated();

    void getLockStat(Chain& lockName, long& lockCount, long &numRdLock, long &numWrLock, long &sumRdDelay, long &sumWrDelay);

 private:

    void lockQueue();
    void unlockQueue();
	
    long* _threadId;
    long* _numRequest;
    ThreadState* _threadState;
    CegoAdminThread** _threadList;
    

    int _poolLimit;
    Chain _adminHostName;
    int _adminPortNo;

    bool _terminated;
    bool _joined;

    ListT<NetHandler*> _requestQueue;

    CegoDatabaseManager* _pDBMng;

    unsigned long _modId;
};

#endif














